@@ -12,6 +12,7 @@ cvrisk.mboost <- function (object, folds = cv(model.weights(object)),
                            fun = NULL, corrected = TRUE, mc.preschedule = FALSE,
                            ...) {
 
+    papply <- match.fun(papply)
     weights <- model.weights(object)
     if (any(weights == 0))
         warning("zero weights")
@@ -72,7 +73,7 @@ cvrisk.mboost <- function (object, folds = cv(model.weights(object)),
     ## use case weights as out-of-bag weights (but set inbag to 0)
     OOBweights <- matrix(rep(weights, ncol(folds)), ncol = ncol(folds))
     OOBweights[folds > 0] <- 0
-    if (all.equal(papply, mclapply) == TRUE) {
+    if (identical(papply, mclapply)) {
         oobrisk <- papply(1:ncol(folds),
                           function(i) try(dummyfct(weights = folds[, i],
                                                    oobweights = OOBweights[, i]),
@@ -118,8 +119,8 @@ print.cvrisk <- function(x, ...) {
     return(invisible(x))
 }
 
-plot.cvrisk <- function(x, ylab = attr(x, "risk"),
-                        xlab = "Number of boosting iterations",
+plot.cvrisk <- function(x, xlab = "Number of boosting iterations",
+                        ylab = attr(x, "risk"),
                         ylim = range(x), main = attr(x, "type"), ...) {
 
     cm <- colMeans(x)
